﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Animation;

namespace WPFDemo
{
    public enum ScalingModalExpandCollapseAnimation
    {
        None = 0,
        ScaleI,
        ScaleO,
        ScaleV,
        ScaleH,
        RotateCW,
        RotateCCW,
        SlideR,
        SlideL,
        SlideT,
        SlideB,
        Fade,
        Twist,
        Skew1,
        Skew2,
        Skew3,
        Skew4,
        Flip,
        Random
    }


    public partial class ScalingModal : UserControl
    {
        private FrameworkElement _panelToDisable;
        private FrameworkElement _innerControl;
        private static ScalingModal _instance;

        protected ScalingModal()
        {
            InitializeComponent();
            IsHitTestVisible = false;
        }

        static ScalingModal()
        {
            _instance = new ScalingModal();
        }


        public ScalingModalExpandCollapseAnimation Animation { get; set; }

        public double BackgroundOpacity
        {
            set { DisableRectangle.Opacity = value; }
        }

        public static ScalingModal Instance
        {
            get { return _instance; }
        }

        public void Initialize(Panel parentPanel, Panel panelToBeDisabled)
        {
            _panelToDisable = panelToBeDisabled;
            Panel parent = parentPanel;
            parent.Children.Add(this);
        }

        public void Expand(FrameworkElement innerControl, ScalingModalExpandCollapseAnimation animationToUse)
        {
            _innerControl = innerControl;

            ModalCanvas.Width = innerControl.Width;
            ModalCanvas.Height = innerControl.Height;
            ModalCanvas.Children.Clear();
            ModalCanvas.Children.Add(innerControl);

            this.IsHitTestVisible = true;

            if (_panelToDisable != null)
                _panelToDisable.IsEnabled = false;

            RunExpandAnimation(animationToUse);
            innerControl.Focus();
        }

        public void Expand(FrameworkElement innerControl)
        {
            Expand(innerControl, Animation);
        }

        public void Collapse(ScalingModalExpandCollapseAnimation animationToUse)
        {
            if (_panelToDisable != null)
                _panelToDisable.IsEnabled = true;
            this.IsHitTestVisible = false;
            RunCollapseAnimation(animationToUse);
        }

        public void Collapse()
        {
            Collapse(Animation);
        }
        
        
        private void RunExpandAnimation(ScalingModalExpandCollapseAnimation animation)
        {
            ResetTransformation();
            if (animation == ScalingModalExpandCollapseAnimation.Random)
                RunExpandAnimation(GetRandomAnimationEnumeration());
            else
                RunAnimation(this, animation.ToString() + "In");
        }

        private void RunAnimation(ScalingModal scalingModal, string key)
        {
            Storyboard s = Resources[key] as Storyboard;
            s.Begin(this);
        }

        private void ResetTransformation()
        {
            RunAnimation(this, "Reset");
        }

        private static ScalingModalExpandCollapseAnimation GetRandomAnimationEnumeration()
        {
            return (ScalingModalExpandCollapseAnimation)(new Random().Next(1, 
                Enum.GetValues(typeof(ScalingModalExpandCollapseAnimation)).Length - 1));
        }

        private void RunCollapseAnimation(ScalingModalExpandCollapseAnimation animation)
        {
            ResetTransformation();
            if (animation == ScalingModalExpandCollapseAnimation.Random)
                RunCollapseAnimation(GetRandomAnimationEnumeration());
            else
                RunAnimation(this, animation.ToString() + "Out");
        }

        private void expandingOpacityAnimation_Completed(object sender, EventArgs e)
        {
            if (_innerControl != null)
                _innerControl.Focus();
        }

        private void collapsingOpacityAnimation_Completed(object sender, EventArgs e)
        {
            _innerControl = null;
            ModalCanvas.Children.Clear();
        }       
    }
}
